CodeQL / Analyze (actions) (push) Canceled after 0s
CodeQL / Analyze (python) (push) Canceled after 0s
pre-commit / pre-commit (push) Canceled after 0s
Multi-Arch CI / setup (push) Canceled after 0s
Publish build_manylinux_x86_64 images / publish_build_manylinux_x86_64 (push) Canceled after 0s
Setup (Multi-Arch) / setup (push) Canceled after 0s
Multi-Arch CI / Linux::${{ fromJSON(needs.setup.outputs.linux_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubi10 image / publish_no_rocm_image_ubi10 (push) Canceled after 0s
Multi-Arch CI / Windows::${{ fromJSON(needs.setup.outputs.windows_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Multi-Arch CI / Manifest Diff (push) Canceled after 0s
TheRock Manifest Diff Report / Generate Manifest Diff Report (push) Canceled after 0s
Multi-Arch CI / CI Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_ocl_rt images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_rocgdb images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Unit Tests / Unit Tests :: ubuntu-24.04 (push) Canceled after 0s
Unit Tests / Unit Tests :: windows-2022 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_media (push) Canceled after 0s
Unit Tests / Unit Tests Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_openmpi (push) Canceled after 0s
63 lines
1.8 KiB
C++
63 lines
1.8 KiB
C++
//===----------------------------------------------------------------------===//
|
|
//
|
|
// Part of the libcu++ Project, under the Apache License v2.0 with LLVM Exceptions.
|
|
// See https://llvm.org/LICENSE.txt for license information.
|
|
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
|
|
// SPDX-FileCopyrightText: Copyright (c) 2025 NVIDIA CORPORATION & AFFILIATES.
|
|
//
|
|
//===----------------------------------------------------------------------===//
|
|
|
|
#include <cuda/bit>
|
|
#include <cuda/std/cassert>
|
|
#include <cuda/std/cstdint>
|
|
#include <cuda/std/type_traits>
|
|
|
|
#include "test_macros.h"
|
|
|
|
template <typename T>
|
|
__host__ __device__ constexpr bool test()
|
|
{
|
|
using nl = cuda::std::numeric_limits<T>;
|
|
constexpr T all_ones = static_cast<T>(~T{0});
|
|
constexpr T half_low = all_ones >> (nl::digits / 2u);
|
|
constexpr T half_high = static_cast<T>(all_ones << (nl::digits / 2u));
|
|
static_assert(cuda::bit_reverse(all_ones) == all_ones);
|
|
static_assert(cuda::bit_reverse(T{0}) == T{0});
|
|
static_assert(cuda::bit_reverse(half_low) == half_high);
|
|
static_assert(cuda::bit_reverse(T{0b11001001}) == (T{0b10010011} << (nl::digits - 8u)));
|
|
static_assert(cuda::bit_reverse(T{T{0b10010011} << (nl::digits - 8u)}) == T{0b11001001});
|
|
unused(all_ones);
|
|
unused(half_low);
|
|
unused(half_high);
|
|
return true;
|
|
}
|
|
|
|
__host__ __device__ constexpr bool test()
|
|
{
|
|
test<unsigned char>();
|
|
test<unsigned short>();
|
|
test<unsigned>();
|
|
test<unsigned long>();
|
|
test<unsigned long long>();
|
|
|
|
test<uint8_t>();
|
|
test<uint16_t>();
|
|
test<uint32_t>();
|
|
test<uint64_t>();
|
|
test<size_t>();
|
|
test<uintmax_t>();
|
|
test<uintptr_t>();
|
|
|
|
#if _CCCL_HAS_INT128()
|
|
test<__uint128_t>();
|
|
#endif // _CCCL_HAS_INT128()
|
|
return true;
|
|
}
|
|
|
|
int main(int, char**)
|
|
{
|
|
assert(test());
|
|
static_assert(test());
|
|
return 0;
|
|
}
|